@@ -1,52 +1,74 @@
+from datetime import datetime, timedelta
+
 import numpy as np
 import pandas as pd
 import requests
 from fbprophet import Prophet
+from sparkles.fbprophet_hacks import suppress_stdout_stderr
 
-# declare variables
+# declare variables for testing
 store = "KT220"
 tank = "7"
 fc_url = "https://fc.bbdev.host3.capspire.com"
 
 
-def decompose_sales(df):
+def decompose_sales(sales_data):
 
-    if df is None:
-        return None
+    # not enough data
+    if sales_data is None or len(sales_data) < 7:
+        return pd.Series([1] * 53), pd.Series([1] * 7)
 
-    if len(df) == 0:
-        return None
+    # fit model for weekly seasonality
+    m = Prophet(
+        changepoint_prior_scale=0.05,
+        daily_seasonality=False,
+        weekly_seasonality=True,
+        yearly_seasonality=True,
+        seasonality_mode="multiplicative",
+    )
+    with suppress_stdout_stderr():
+        m.fit(sales_data)
 
-    else:
-        m = Prophet(
-            changepoint_prior_scale=0.05,
-            daily_seasonality=False,
-            weekly_seasonality=True,
-            yearly_seasonality=True,
-            seasonality_mode="multiplicative",
+    # calculate weekly seasonality
+    future = m.make_future_dataframe(periods=1, freq="1D")
+    forecast = m.predict(future)
+    forecast_reduced = forecast.loc[:, ["ds", "yearly", "weekly"]]
+    forecast_reduced["dow"] = forecast_reduced.ds.dt.weekday
+    forecast_reduced["doy"] = forecast_reduced.ds.dt.dayofyear
+    forecast_reduced["woy"] = forecast_reduced.ds.dt.week
+
+    weekly_seasonality = forecast_reduced.groupby("dow")["weekly"].mean() + 1
+
+    if len(weekly_seasonality) < 7:
+        return pd.Series([1] * 53), pd.Series([1] * 7)
+
+    if len(sales_data) < 350:
+        return pd.Series([1] * 53), weekly_seasonality
+
+    # if nearly a full year, or more of data exists, calculate yearly seasonality
+    forecast_reduced = forecast_reduced.iloc[-366:]
+    forecast_reduced.sort_values(by=["woy", "dow"], inplace=True)
+    year_trend = forecast_reduced.groupby("woy").mean()
+    year_trend = year_trend["yearly"] + 1
+
+    # ensure 53 weeks are calculated by wrapping week values to end of the year
+    if len(year_trend) < 53:
+        delta = 53 - len(year_trend)
+        yearly_seasonality = pd.concat(
+            [year_trend, year_trend[:delta]], ignore_index=True
         )
 
-        m.fit(df)
-        future = m.make_future_dataframe(periods=1, freq="1D")
-        forecast = m.predict(future)
-        forecast_reduced = forecast.loc[:, ["ds", "yearly", "weekly"]]
-        forecast_reduced["dow"] = forecast_reduced.ds.dt.weekday
-        forecast_reduced["doy"] = forecast_reduced.ds.dt.dayofyear
-        forecast_reduced["woy"] = forecast_reduced.ds.dt.week
-        week_trend = forecast_reduced.groupby("dow")["weekly"].mean() + 1
-        forecast_reduced = forecast_reduced.iloc[-366:]
-        forecast_reduced.sort_values(by=["woy", "dow"], inplace=True)
-        year_trend = forecast_reduced.groupby("woy").mean()
-        year_trend = year_trend["yearly"] + 1
-        if len(year_trend) < 53:
-            delta = 53 - len(year_trend)
-            yearly_seasonality = pd.concat([year_trend, year_trend[:delta]])
-    return yearly_seasonality, week_trend
+    if len(weekly_seasonality) == 7 and len(yearly_seasonality) == 53:
+        return yearly_seasonality, weekly_seasonality
+
+    return pd.Series([1] * 53), pd.Series([1] * 7)
 
 
-# tank history decomposition for daily trends
 def decompose_tank(tank_history):
 
+    if tank_history is None or len(tank_history) < 48:
+        return generic_hh_seasonality
+
     m = Prophet(
         changepoint_prior_scale=0.05,
         daily_seasonality=True,
@@ -54,23 +76,26 @@ def decompose_tank(tank_history):
         yearly_seasonality=False,
         seasonality_mode="multiplicative",
     )
+    with suppress_stdout_stderr():
+        m.fit(tank_history)
 
-    m.fit(tank_history)
     future = m.make_future_dataframe(periods=1, freq="30min")
     forecast = m.predict(future)
-    forecast_reduced = forecast.loc[:, ["ds", "daily", "daily_lower", "daily_upper"]]
+
+    forecast_reduced = forecast.loc[:, ["ds", "daily"]]
     forecast_reduced["time"] = forecast_reduced["ds"].dt.time
     forecast_reduced.drop(columns="ds", inplace=True)
     day_trend = forecast_reduced.groupby("time").mean()
     day_trend.sort_values(by=["time"], inplace=True)
 
-    for field in ["daily", "daily_lower", "daily_upper"]:
-        day_trend.loc[:, field] = day_trend.loc[:, field] + 1
+    if len(day_trend) != 48:
+        return generic_hh_seasonality
 
-    return day_trend
+    hh_seasonality = day_trend["daily"] + 1
+
+    return hh_seasonality
 
 
-# holiday decomposition - needs improvement for consistent date mapping
 def decompose_holiday(input):
     if type(input) != pd.core.frame.DataFrame:
         return print("error - input is not a DataFrame")
@@ -101,7 +126,8 @@ def decompose_holiday(input):
             seasonality_mode="multiplicative",
         )
         m.add_country_holidays(country_name="US")
-        m.fit(input)
+        with suppress_stdout_stderr():
+            m.fit(input)
         future = m.make_future_dataframe(periods=1, freq="1D")
         forecast = m.predict(future)
         forecast_reduced = forecast.loc[:, ["ds", "holidays"]]
@@ -178,7 +204,7 @@ generic_hh_seasonality = [
 ]
 
 
-# # request sales data
+# request sales data
 # def get_sales_history(
 #     store_number: str, tank_id: str,
 # ):
@@ -194,19 +220,52 @@ generic_hh_seasonality = [
 #     return r.json()
 
 
-# from tank_forecaster import validation
-#
+from tank_forecaster import validation
+
 #
 # def main():
 #     x = get_sales_history(store_number=store, tank_id=tank)
 #     y = validation.format_sales(x)
 #     z = decompose_sales(y)
 #     print(z[0], z[1])
-#     print(len(z[0]))
-#
-#
-# print(generic_yearly_seasonality)
-# print(len(generic_yearly_seasonality))
+#     print(len(z[0]), len(z[1]))
+
+tank_url = "https://bbdev.host3.capspire.com/api/tms"
+
+
+def now_utc(exact: bool = False) -> datetime:
+    import pytz
+
+    date_time_now = datetime.now(pytz.utc).replace(tzinfo=None)
+    if exact:
+        return date_time_now
+    return date_time_now.replace(second=0, microsecond=0)
+
+
+def get_readings(
+    store_number: str,
+    tank_id: str,
+    start_date: datetime = now_utc() - timedelta(days=16),
+):
+    parameters = {
+        "store_number": store_number,
+        "tank_id": tank_id,
+        "start_date": start_date,
+        "system_psk": "lol",
+    }
+    r = requests.post(url=f"{tank_url}/tank_inventory/readings", params=parameters)
+    return r.json()
+
+
+from tank_forecaster import validation
+
+
+def main():
+    x = get_readings(store_number=store, tank_id=tank)
+    y = validation.format_tank(x)
+    z = decompose_tank(y)
+    print(z)
+
 
 if __name__ == "__main__":
     main()
